from zoodle.moodle import Moodle, CredentialsError
from zoodle.command import Command

class CheckLoginCommand(Command):
    def __init__(self):
        pass

    @classmethod
    def help_text(cls):
        return "confirm credentials can access moodle"

    def parse_arguments(self, args):
        return vars(self.argument_parser().parse_args(args))

    def main(self, args):
        options = self.parse_arguments(args)

        # Load configuration from file if requested
        if options['configfile'] is not None:
            config_options = self.load_configuration(options['configfile'])
            options.update(config_options)

        self.check_login_details(options)

        try:
            Moodle(options['baseurl'], options['username'], options['password'])
            print("credentials OK")
        except CredentialsError:
            print("credentials incorrect")
        



from argparse import ArgumentParser
from configparser import RawConfigParser
import getpass
import abc

class Command:
    """
    Superclass for commands called from CLI dispatcher
    """

    __metaclass__ = abc.ABCMeta

    @classmethod
    def help_text(cls):
        return "base command"

    def argument_parser(self):
        parser = ArgumentParser(
            epilog='(c) 2015 Peadar Grant',
            )
        parser.description = self.help_text()
        parser.add_argument('-c', '--configfile', help='config file to load', default=None)
        parser.add_argument('-b', '--baseurl', help='base URL')
        parser.add_argument('-u', '--username', help='username (will prompt if not supplied)')
        parser.add_argument('-p', '--password', help='password (will prompt if not supplied)')
        return parser

    def load_configuration(self, filename):
        """
        Loads configuration from file
        """
        config = RawConfigParser()
        config.read(filename)
        options = {}
        for key in config['default']:
            options[key] = config['default'][key]
        return options    

    def check_login_details(self, options):
        """
        Confirm that the username and password are available, prompting if required.
        """
        if options['baseurl'] is None:
            options['baseurl'] = input('Base URL: ')
        if options['username'] is None:
            options['username'] = input('Username: ')
        if options['password'] is None:
            options['password'] = getpass.getpass()



import datetime
import os

import jinja2

from zoodle.command import Command
from zoodle.moodle import Moodle

class UserListCommand(Command):
    def __init__(self):
        pass

    @classmethod
    def help_text(cls):
        return "query and template user list for course"
    
    def parse_arguments(self, args):
        parser = self.argument_parser()
        parser.add_argument('courseid', help='course ID number (from URL, not course code)')
        parser.add_argument('-T', '--template', help='template name', default="list.txt")
        parser.add_argument('-t', '--title', help='title (usage depends on template)', default='Class list')
        parser.add_argument('--groupid', help='group ID number to filter by', default=None)
        parser.add_argument('--roleid', help='role ID number to filter by', default=None)
        parser.add_argument('--templatedir', help='template directory', default=None)
        return vars(parser.parse_args(args))

    def main(self, args):

        options = self.parse_arguments(args)

        # Load configuration from file if requested
        if options['configfile'] is not None:
            config_options = self.load_configuration(options['configfile'])
            options.update(config_options)

        self.check_login_details(options)

        moodle = Moodle(options['baseurl'], options['username'], options['password'])
        tree = moodle.get_user_tree(options['courseid'], options['groupid'], options['roleid'])

        # Get all students enrolled
        students = tree.xpath("//tr[@class='userinforow']/td[@class='field col_userdetails cell c0']/div[@class='subfield subfield_userfullnamedisplay']/text()")

        # Template setup
        this_dir = os.path.split(__file__)[0]
        if options['templatedir'] is not None:
            templates_dir = options['templatedir']
        else:
            templates_dir = os.path.join(this_dir, 'templates')
        loader = jinja2.FileSystemLoader(templates_dir)
        environment = jinja2.Environment(loader=loader)
        template = environment.get_template(options['template'])
        template_vars = {
            "course": tree.xpath("//h1/text()")[0],
            "code": tree.xpath("(//ul[@class='breadcrumb']/li/a)[2]/text()")[0],
            "title": options['title'],
            "timestamp": datetime.date.today().strftime("%B %d, %Y"),
            "students": students,
            "number_of_students": len(students),
            "group": tree.xpath("//select[@id='id_filtergroup']/option[@selected='selected']/text()")[0],
            "role": tree.xpath("//select[@name='role']/option[@selected='selected']/text()")[0],
        }

        # Send template to output
        print(template.render(template_vars))



from requests import session
from lxml import html

class Moodle:
    """
    Moodle client to return HTML pages for specific requests
    """

    def __init__(self, baseurl, username, password):
        """
        Create a new moodle session with given credentials
        """
        super().__init__()
        self.baseurl = baseurl
        self.session = session()
        login_payload = {
            'username': username,
            'password': password,
        }
        response = self.session.post('%s/login/index.php' % baseurl, data=login_payload)
        if "not logged in" in response.text:
            raise CredentialsError("username/password incorrect for user %s" % username)


    def available_courses(self):
        """
        Return the course list
        """
        response = self.session.get('%s/' % self.baseurl)
        tree = html.fromstring(response.content)
        courses = tree.xpath("//div[contains(@class, 'coursebox')]//h3/a/text()")
        return courses
        
    def get_user_tree(self, courseid, groupid=None, roleid=None):
        """
        Return the HTML user tree of enrolled participants
        """
        search_params = {
            'id': courseid,
            'perpage': '10000'
            }
        if groupid is not None:
            search_params['filtergroup'] = groupid
        if roleid is not None:
            search_params['role'] = roleid
        response = self.session.get(
            '%s/enrol/users.php' % ( self.baseurl ),
            params=search_params)
        return html.fromstring(response.content)
    
class CredentialsError(Exception):
    def __init__(self, value):
        super().__init__()
        self.value = value
    def __str__(self):
        return repr(self.value)
    



#!/usr/bin/env python3
# Class list query tool for Moodle
# Copyright (C) 2015 Peadar Grant
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
from argparse import ArgumentParser

from zoodle.commands.checklogin import *
from zoodle.commands.userlist import *
from zoodle.commands.courses import *

def main():
    """
    main command dispatcher
    """
    # Supported commands
    available_commands = {
        "courses": CoursesCommand,
        "userlist": UserListCommand,
        "checklogin": CheckLoginCommand,
        }

    # Build explanation string for help display
    command_explanation = ""
    for command in available_commands:
        command_explanation = ( command_explanation +
                                "  " +
                                command.ljust(12) +
                                available_commands[command].help_text()
                                + "\n" )
    # following method from:
    # http://chase-seibert.github.io/blog/2014/03/21/python-multilevel-argparse.html
    parser = ArgumentParser(
        usage=""" zoodle <command> [<args>]
available commands:

%s

help on individual commands: zoodle <command> -h

""" % command_explanation)
    parser.add_argument('command', help='command to run')
    args = parser.parse_args(sys.argv[1:2])
    if args.command not in available_commands:
        print("unrecognized command")
        parser.print_help()
        exit(1)

    command = available_commands[args.command]()
    command.main(sys.argv[2:])

if __name__ == '__main__':
    main()



from zoodle.moodle import Moodle, CredentialsError
from zoodle.command import Command

class CoursesCommand(Command):
    def __init__(self):
        pass

    @classmethod
    def help_text(cls):
        return "list all accessible courses"
    
    def parse_arguments(self, args):
        return vars(self.argument_parser().parse_args(args))
    
    def main(self, args):
        options = self.parse_arguments(args)

        if options['configfile'] is not None:
            config_options = self.load_configuration(options['configfile'])
            options.update(config_options)

        self.check_login_details(options)
        moodle = Moodle(options['baseurl'], options['username'], options['password'])        
        courses = moodle.available_courses()
        for course in courses:
            print(course)
